/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/callback/OperationCallback.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc. 
 * All rights reserved.
 */
package com.netapp.common.callback;

import java.util.List;

import com.netapp.common.flow.api.OperationState;
import com.netapp.common.logging.LogMessage;

/**
 * The OperationCallback interface is implemented within the client.
 * Each operation that is registered to process the callback will invoke
 * at least one of these methods when new data is available on the server
 * about the operation in question. 
 * <br />
 * The 'id' can be either an operationId or a clientRequestId.
 * <br />
 * When implementing this interface, ensure that no real processing is done inside
 * of the implementation. The callback should only set data and any extra processing
 * should be done outside of the callback so as to not impact the performance of the
 * callback threads.
 * 
 * @author joshb
 *
 */
public interface OperationCallback {

    /**
     * A listing of generic log messages for the given operation.
     * 
     * @param id
     * @param messages
     */
    void message(String id, List<LogMessage> messages);

    /**
     * A listing of PROGRESS log messages for the given operation. These log messages
     * differ from the generic log messages by having the LogLevel set to PROGRESS.
     * 
     * @param id
     * @param progress
     */
    void progress(String id, List<LogMessage> progress);

    /**
     * The current state of the operation. The same state may be set multiple times.
     * 
     * @param id
     * @param state
     */
    void status(String id, OperationState state);
    
    /**
     * An id that did not finish before the callback for the server was terminated
     * early.
     */
    void unprocessedId(String id);
}
