/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/encryption/SymmetricEncryption.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc. 
 * All rights reserved.
 */
package com.netapp.common.encryption;

import java.security.Key;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.spec.SecretKeySpec;

/**
 * Provide an easy, consistent way to encrypt and decrypt using a symmetric key algorithm.
 * 
 * @author joshb
 *
 */
public class SymmetricEncryption {

    protected SymmetricEncryption() {
    }

    /**
     * Given an algorithm and secret key, encrypt the incoming string and return it as a byte array. 
     * All encryption exceptions are wrapped and rethrown.
     * 
     * @param algorithm
     * @param secret
     * @param encrypt
     * @return Encrypted string for encrypt, using secret as the key.
     * @throws EncryptionException
     */
    public static byte[] encrypt(String algorithm, byte[] secret, String encrypt) throws EncryptionException {

        try {
            Cipher cipher = getCipher(algorithm, secret, Cipher.ENCRYPT_MODE);

            return cipher.doFinal(encrypt.getBytes());
        }
        catch (IllegalBlockSizeException e) {
            throw new EncryptionException(e);
        }
        catch (BadPaddingException e) {
            throw new EncryptionException(e);
        }
    }

    /**
     * Given an algorithm and secret key, decrypt the incoming byte array and return it as a string. 
     * All encryption exceptions are wrapped and rethrown.
     * 
     * @param algorithm
     * @param secret
     * @param decrypt
     * @return Plaintext of decrypt using secret as the key
     * @throws EncryptionException
     */
    public static String decrypt(String algorithm, byte[] secret, byte[] decrypt) throws EncryptionException {

        try {
            Cipher cipher = getCipher(algorithm, secret, Cipher.DECRYPT_MODE);

            return new String(cipher.doFinal(decrypt));
        }
        catch (IllegalBlockSizeException e) {
            throw new EncryptionException(e);
        }
        catch (BadPaddingException e) {
            throw new EncryptionException(e);
        }
    }

    /**
     * A consistent way to retrieve a Cipher for a given algorithm, secret and mode.
     * 
     * @param algorithm
     * @param secret
     * @param cipherMode
     * @return Cipher for the given algorithm, secret, and mode.
     * @throws EncryptionException
     */
    public static Cipher getCipher(String algorithm, byte[] secret, int cipherMode) throws EncryptionException {
        Key key = new SecretKeySpec(secret, algorithm);
        return Encryption.getCipher(key, algorithm, cipherMode);
    }
}
