/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/flow/CheckpointTemplate.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.flow;

import com.netapp.common.MsgKey;
import com.netapp.common.logging.NALogger;

/**
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
class CheckpointTemplate implements ExecutionStep {
    private static final NALogger logger = NALogger.getLogger(ExecutionStep.class);
    
    private String name;
    private String abortPoint;
    
    CheckpointTemplate(String name, String abortPoint) {
        this.name = name;
        this.abortPoint = abortPoint;
    }
    
    public boolean hasName() {
        return this.name != null && !this.name.equals("");
    }
    public String getName() {
        return this.name;
    }
    
    public boolean hasAbortPoint() {
        return this.abortPoint != null && !this.abortPoint.equals("");
    }
    public String getAbortPoint() {
        return this.abortPoint;
    }
    
    @Override
    public String toString() {
        return "Checkpoint: " + ((this.name) == null ? "" : this.name) + ((this.abortPoint == null) ? "" : "Abort " + this.abortPoint);
    }

    public void execute(StackFrame stackFrame) {
        // If we've deferred a failure to checkpoint, we fail that now.
        if (FailureStrategy.CHECKPOINT.equals(stackFrame.getContext().getFailureStrategy())) {
            stackFrame.getContext().setFailureStrategy(FailureStrategy.IMMEDIATE);
            throw new FlowException(logger.debug(MsgKey.FLOW_OPERATION_CHECKPOINT_DELAYED_FAILURE));
        }
        
        logger.internal("Executing checkpoint %s", this.name);
        try {
            // If we're not running the abort stack, the name of this checkpoint
            // is the normal one. If we're aborting, the name is the abort point.
            if (!stackFrame.getContext().isAborting()) {
                if (this.hasName()) {
                    stackFrame.getContext().setCheckpoint(this.name);
                }
                
                if (this.hasAbortPoint()) {
                    stackFrame.getContext().setAbortCheckpoint(this.abortPoint);
                }
            }
            else {
                if (this.hasName()) {
                    stackFrame.getContext().setAbortCheckpoint(this.name);
                }
            }
            
            // We only want to reset the retry count if we hit a named checkpoint
            if (this.hasName()) {
                stackFrame.getContext().resetRetryCount();
            }
            
            stackFrame.getContext().loadPersistentRegisters();
            stackFrame.getContext().persist();
        }
        // CHECKSTYLE:OFF
        catch (RuntimeException e) {
        // CHECKSTYLE:ON
            logger.warn(MsgKey.FLOW_OPERATION_CHECKPOINT_EXCEPTION, e);
        }
    }
}
