/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/flow/Result.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.flow;

import java.io.Serializable;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.netapp.common.flow.api.OperationState;
import com.netapp.common.flow.api.OperationUpdateTag;
import com.netapp.common.logging.LogMessage;

/**
 * Structure representing the external results of an operation.
 * 
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class Result implements Serializable {
    /**
     * 
     */
    private static final long serialVersionUID = 1L;
    
    /**
     * Workflow engine id of this operation.
     */
    protected String id;
    
    /**
     * Secondary client id of this operation.
     */
    protected String clientRequestId;
    
    /**
     * Operation type of this operation.
     */
    protected String type;
    
    /**
     * Current state of this operation.
     */
    protected OperationState state;
    
    /**
     * Time the operation was started
     */
    protected Date createTime;
    
    /**
     * Last checkpoint time of this operation.
     */
    protected Date lastActiveTime;
    
    /**
     * Last time a log message or the state changed.
     */
    protected OperationUpdateTag lastUpdate;
    
    /**
     * List of logged messages for this operation.
     * Map of output values for this operation. Only filled in
     * if {@link #state} is {@link OperationState#COMPLETE COMPLETE},
     * {@link OperationState#FAILED FAILED} or 
     * {@link OperationState#INCOMPLETE INCOMPLETE}.
     */
    protected List<LogMessage> messages;
    
    /**
     * Map of output values for this operation. Only filled in
     * if {@link #state} is {@link OperationState#COMPLETE COMPLETE}.
     */
    protected Map<String, Object> output;
    
    /**
     * Error provided by an exception thrown that implements {@link ClientAccessibleErrorProvider}.
     * This object will be returned to the client via the OperationResult.
     * 
     * Only filled in if an exception was thrown that implements {@link ClientAccessibleErrorProvider}.
     */
    protected Object clientAccessibleError;
    
    public Result() {
        
    }
    
    public Result(OperationId id) {
        this.id = id.getId();
    }
    
    public void setId(String id) {
        this.id = id;
    }
    
    /**
     * @see com.netapp.common.flow.api.OperationResult#getId()
     */
    public String getId() {
        return id;
    }
    
    public void setClientRequestId(String clientRequestId) {
        this.clientRequestId = clientRequestId;
    }
    
    public String getClientRequestId() {
        return this.clientRequestId;
    }
    
    /**
     *
     * @param type
     */
    public void setType(String type) {
        this.type = type;
    }
    
    /**
     * @see com.netapp.common.flow.api.OperationResult#getType()
     */
    public String getType() {
        return this.type;
    }
    
    /**
     * Set the create time of this operation
     * @param createTime Time this operation was created
     */
    public void setCreateTime(Date createTime) {
        this.createTime = createTime;
    }
    
    /**
     * Get the create time of this operation
     * @return Time this operation was created
     */
    public Date getCreateTime() {
        return this.createTime;
    }
    
    /**
     * Set the last checkpoint time of this operation.
     * @param completionTime time this operation checkpointed.
     */
    public void setLastActiveTime(Date completionTime) {
        this.lastActiveTime = completionTime;
    }
    
    /**
     * @see com.netapp.common.flow.api.OperationResult#getLastActiveTime()
     */
    public Date getLastActiveTime() {
        return lastActiveTime;
    }

    public void setOutput(Map<String, Object> output) {
        this.output = output;
    }
    
    /**
     * Retrieve the outputs from this operation.
     * Only valid if {@link #getState()} is {@link OperationState#COMPLETE COMPLETE}
     * @return Map of output names and values
     */
    public Map<String, Object> getOutput() {
        return output;
    }
    
    public void setState(OperationState state) {
        this.state = state;
    }

    /**
     * @see com.netapp.common.flow.api.OperationResult#getState()
     */
    public OperationState getState() {
        return state;
    }
    
    /**
     * @see com.netapp.common.flow.api.OperationResult#getMessages()
     */
    public List<LogMessage> getMessages() {
        return this.messages;
    }
    
    public void setMessages(List<LogMessage> messages) {
        this.messages = messages;
    }
    
    /**
     * Add an output to this operation.
     * @param name output name
     * @param value output value
     */
    protected void addOutput(String name, Object value) {
        if (output == null) {
            output = new HashMap<String, Object>();
        }
        output.put(name, value);
    }
    
    /**
     * @see com.netapp.common.flow.api.OperationResult#getLastUpdate()
     */
    public OperationUpdateTag getLastUpdate() {
        return lastUpdate;
    }

    public void setLastUpdate(OperationUpdateTag lastUpdate) {
        this.lastUpdate = lastUpdate;
    }
    
    public Object getClientAccessibleError() {
        return this.clientAccessibleError;
    }
    
    public void setClientAccessibleError(Object clientAccessibleError) {
        this.clientAccessibleError = clientAccessibleError;
    }
}
