/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/flow/soap/BlockingResultDelegate.java#1 $
 * 
 * Copyright (c) 2008 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.flow.soap;

import java.util.HashMap;
import java.util.Map;

import com.netapp.common.flow.Engine;
import com.netapp.common.flow.LifecycleAdapter;
import com.netapp.common.flow.LifecycleListener;
import com.netapp.common.flow.OperationId;
import com.netapp.common.flow.Result;
import com.netapp.common.flow.api.OperationState;
import com.netapp.common.logging.NALogger;

/**
 * ResultDelegate that only returns an answer once the operation is in a complete state,
 * or the waiting thread is interrupted.
 * 
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class BlockingResultDelegate implements ResultDelegate {
    private static final NALogger logger = NALogger.getLogger(BlockingResultDelegate.class);
    private Map<String, Object> monitors = new HashMap<String, Object>();
    private LifecycleListener listener;
    private long timeoutMillis;
    
    /**
     * Construct a new blocking result delegate with an infinite timeout.
     */
    public BlockingResultDelegate() {
        this.timeoutMillis = 0;
    }
    
    /**
     * Construct a new blocking result delegate with the specified timeout.
     * @param timeoutMillis
     */
    public BlockingResultDelegate(long timeoutMillis) {
        this.timeoutMillis = timeoutMillis;
    }
    
    public void setTimeout(long timeoutMillis) {
        this.timeoutMillis = timeoutMillis;
    }
    
    /**
     * @see com.netapp.common.flow.soap.ResultDelegate#getResult(com.netapp.common.flow.Engine, java.lang.String)
     */
    public Result getResult(Engine engine, String id) {
        Result baseResult = engine.operationResult(id);
        
        if (!OperationState.RUNNING.equals(baseResult.getState())) {
            return baseResult;
        }
        
        synchronized (this) {
            if (this.listener == null) {
                this.listener = new Listener();
                engine.addLifecycleListener(this.listener);
            }
        }
        
        Object monitor = null;
        String key = baseResult.getId();
        synchronized (this.monitors) {
            monitor = this.monitors.get(key);
            if (monitor == null) {
                monitor = new Object();
                this.monitors.put(key, monitor);
            }
        }
        
        synchronized (monitor) {
            baseResult = engine.operationResult(key);
            if (OperationState.RUNNING.equals(baseResult.getState())) {
                try {
                    monitor.wait(timeoutMillis);
                }
                catch (InterruptedException e) {
                    Thread.currentThread().interrupt();
                }
                baseResult = engine.operationResult(key);
            }
        }
        
        synchronized (this.monitors) {
            this.monitors.remove(key);
        }
        return baseResult;
    }

    private class Listener extends LifecycleAdapter {
        /**
         */
        public void operationCompleted(OperationId id, String clientRequestId) {
            logger.internal("Notifying for operation complete %s %s", id, clientRequestId);
            Object monitor = null;
            synchronized (BlockingResultDelegate.this.monitors) {
                monitor = BlockingResultDelegate.this.monitors.get(id.getId());
            }
            
            if (monitor != null) {
                synchronized (monitor) {
                    monitor.notifyAll();
                }
            }
        }
    }
}
