/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/gwt/GWTRPCDispatcher.java#1 $
 * 
 * Copyright (c) 2009 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.common.gwt;

import java.io.Serializable;
import java.lang.reflect.Method;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.BeanFactoryUtils;
import org.springframework.web.context.WebApplicationContext;
import org.springframework.web.context.support.WebApplicationContextUtils;

import com.google.gwt.user.client.rpc.IsSerializable;
import com.google.gwt.user.client.rpc.SerializationException;
import com.google.gwt.user.server.rpc.RPC;
import com.google.gwt.user.server.rpc.RPCRequest;
import com.google.gwt.user.server.rpc.RemoteServiceServlet;
import com.google.gwt.user.server.rpc.SerializationPolicy;
import com.google.gwt.user.server.rpc.impl.ServerSerializationStreamReader;

public class GWTRPCDispatcher extends RemoteServiceServlet {

    private static final ThreadLocal<HttpServletRequest> THREAD_LOCAL_REQUEST = new ThreadLocal<HttpServletRequest>();
    
    private static final ThreadLocal<HttpServletResponse> THREAD_LOCAL_RESPONSE = new ThreadLocal<HttpServletResponse>();
    
    private static final SerializationPolicy SERIALIZATION_POLICY = new SerializationPolicy() {

        @Override
        public boolean shouldDeserializeFields(Class<?> clazz) {
            return IsSerializable.class.isAssignableFrom(clazz) || Serializable.class.isAssignableFrom(clazz);
        }

        @Override
        public boolean shouldSerializeFields(Class<?> clazz) {
            return IsSerializable.class.isAssignableFrom(clazz) || Serializable.class.isAssignableFrom(clazz);
        }

        @Override
        public void validateDeserialize(Class<?> clazz) throws SerializationException {
        }

        @Override
        public void validateSerialize(Class<?> clazz) throws SerializationException {
        }
        
    };
    
    @Override
    protected SerializationPolicy doGetSerializationPolicy(HttpServletRequest request, String moduleBaseURL, String strongName) {
        return SERIALIZATION_POLICY; 
    }
    
    @Override
    public String processCall(String payload) throws SerializationException {
        Exception caught = null;
        Method method = null;
        try {
            THREAD_LOCAL_REQUEST.set(this.getThreadLocalRequest());
            THREAD_LOCAL_RESPONSE.set(this.getThreadLocalResponse());

            ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
            
            ServerSerializationStreamReader serverSerializationStreamReader = new ServerSerializationStreamReader(classLoader, this);
            serverSerializationStreamReader.prepareToRead(payload);
            
            String serviceInterfaceName = serverSerializationStreamReader.readString();
            Class<?> serviceInterfaceClass = Class.forName(serviceInterfaceName, false, classLoader);
            
            WebApplicationContext webApplicationContext = WebApplicationContextUtils.getWebApplicationContext(this.getServletContext());
            Object serviceImplementation = BeanFactoryUtils.beanOfTypeIncludingAncestors(webApplicationContext, serviceInterfaceClass);
            
            RPCRequest rpcRequest = RPC.decodeRequest(payload, serviceImplementation.getClass(), this);
            method = rpcRequest.getMethod();
            
            return RPC.invokeAndEncodeResponse(serviceImplementation, method, rpcRequest.getParameters(), rpcRequest.getSerializationPolicy());
        }
        catch (ClassNotFoundException e) {
            caught = e;
        }
        finally {
            THREAD_LOCAL_REQUEST.set(null);
            THREAD_LOCAL_RESPONSE.set(null);
        }
        
        return RPC.encodeResponseForFailure(method, caught);
    }
    
    public static HttpServletRequest getCurrentRequest() {
        return THREAD_LOCAL_REQUEST.get();
    }
    
    public static HttpServletResponse getCurrentResponse() {
        return THREAD_LOCAL_RESPONSE.get();
    }
}
