/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/platform/darwin/DFResultConstructor.java#1 $
 *
 * Copyright (c) 2008 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.common.platform.darwin;

import java.io.File;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.netapp.common.MsgKey;
import com.netapp.common.execution.ShellResult;
import com.netapp.common.logging.NALogger;
import com.netapp.common.platform.PlatformException;
import com.netapp.common.platform.unix.DFResult;
import com.netapp.common.platform.unix.DFResult.FileInfo;

public class DFResultConstructor extends com.netapp.common.platform.unix.DFResultConstructor {

    private static final NALogger logger = NALogger.getLogger(DFResultConstructor.class);
    /**
     * Matches df output on OS X:
     * Filesystem                                   512-blocks       Used     Avail Capacity  Mounted on
     * /dev/disk1s10                                 251374080   67792488 183581592    27%    /Volumes/Untitled 1
     * pufferfish.rtp.netapp.com:/vol/vmware_swdist  125829120   79161968  46667152    63%    /Volumes/pufferfish.rtp.netapp.com
     */
    public static final Pattern PATTERN = Pattern.compile("(\\S+)\\s+\\S+\\s+\\S+\\s+\\S+\\s+\\S+\\s+(\\S.*)\\s*");
    public DFResultConstructor() {
        super(1, 1);
    }
    
    @Override
    protected DFResult doConstruct(List<File> fileList, ShellResult shellResult) {
        DFResult dfResult = new DFResult();
        int outputLineIndex = 1;
        for (File file : fileList) {
            String dfline = shellResult.getStdOut().get(outputLineIndex++);
            
            Matcher matcher = PATTERN.matcher(dfline);
            
            if (!matcher.matches()) {
                String error = logger.error(MsgKey.PLATFORM_ERROR_PARSING_D_F_OUTPUT, shellResult.getStdOut().toString());
                throw new PlatformException(error);
            }
            String fileSystem = matcher.group(1);
            String mountPoint = matcher.group(2);

            String filer = null;
            String export = null;
            // This is kind of a guess if it looks like NFS
            // If we use this for more than dev, I think we need a better way to
            // get this than df. CIFS and AFP need more than we can get from this.
            if (fileSystem.contains(":")) {
                String[] split = fileSystem.split(":");
                if (split.length != 2) {
                    throw new PlatformException(
                        logger.error(MsgKey.PLATFORM_ERROR_PARSING_D_F_OUTPUT, shellResult.getStdOut().toString()));
                }
                filer = split[0];
                export = split[1];
            }
            else if (!fileSystem.startsWith("/dev/disk")) {
                // CIFS or AFP - We don't really know what to do with those yet
                throw new PlatformException(
                    logger.error(MsgKey.PLATFORM_ERROR_PARSING_D_F_OUTPUT, shellResult.getStdOut().toString()));
            }
            dfResult.addFileInfo(file, new FileInfo(fileSystem, new File(mountPoint), filer, export));
        }
        return dfResult;
    }
}
