/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/platform/hpux/OperatingSystemImpl.java#1 $
 *
 * Copyright (c) 2006 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.platform.hpux;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.netapp.common.MsgKey;
import com.netapp.common.execution.ShellCommand;
import com.netapp.common.execution.ShellRunnable;
import com.netapp.common.logging.NALogger;
import com.netapp.common.platform.DeviceAnalyzer;
import com.netapp.common.platform.FileSystemMount;
import com.netapp.common.platform.PartitionTable;
import com.netapp.common.platform.PlatformException;
import com.netapp.common.util.FileUtils;

public class OperatingSystemImpl extends com.netapp.common.platform.unix.OperatingSystemImpl {
    private static final NALogger logger = NALogger.getLogger(OperatingSystemImpl.class);
    /**
     * /dev/PCLVMDG1/lvol1 /mnt/stonefish/pclvm/control vxfs ioerror=mwdisable,delaylog,nodatainlog,dev=40010001 0 0 1234352435
     */
    private static final Pattern MNTTAB_PATTERN = 
        Pattern.compile("(?:(\\S+):)?(\\S+) (\\S+) (\\S+) (\\S+) \\d+ \\d+ \\d+.*");
    
    private static final int MNTTAB_SERVER_GROUP = 1;
    private static final int MNTTAB_DEVICE_GROUP = 2;
    private static final int MNTTAB_MOUNT_POINT_GROUP = 3;
    private static final int MNTTAB_TYPE_GROUP = 4;
    private static final int MNTTAB_OPTIONS_GROUP = 5;
    
    public OperatingSystemImpl() {
        super();
    }

    @Override
    public String getLibraryEnvironmentName() {
        return "SHLIB_PATH";
    }

    @Override
    public DeviceAnalyzer createDeviceAnalyzer() {
        return new HPUXDeviceAnalyzer();
    }

    @Override
    protected ShellRunnable createDFCommand(List<File> fileList) {
        ShellCommand shellCommand = new ShellCommand();
        
        // HP-UX's df command doesn't  respect the ordering of arguments, 
        // it'll reorder them at will so the output won't match the input.
        // Use xargs to force it to run a separate DF command for each
        // argument.
        shellCommand.append("echo");
        for (File file : fileList) {
            shellCommand.append(file.getAbsolutePath());
        }
        shellCommand.append("|", "xargs", "-n", "1", "df", "-n");
        
        return shellCommand;
    }

    @Override
    protected DFResultConstructor createDFResultConstructor() {
        return new com.netapp.common.platform.hpux.DFResultConstructor();
    }

    @Override
    public PartitionTable readPartitionTable(File device) {
        throw new UnsupportedOperationException();
    }
    
    @Override
    public Collection<FileSystemMount> getMountedFilesystems() {
        List<Matcher> mnttab = null;
        try {
            mnttab = FileUtils.matchLines(MNTTAB_PATTERN, getMnttabFile());
        }
        catch (IOException e) {
            throw new PlatformException(
                logger.debug(MsgKey.PLATFORM_MOUNT_TABLE_READ_FAILED, e.getLocalizedMessage()),
                e);
        }
        
        List<FileSystemMount> results = new ArrayList<FileSystemMount>();
        for (Matcher m : mnttab) { 
            String type = m.group(MNTTAB_TYPE_GROUP);
            if ("autofs".equals(type)) {
                continue;
            }
            String server = m.group(MNTTAB_SERVER_GROUP);
            String device = m.group(MNTTAB_DEVICE_GROUP);
            String options = m.group(MNTTAB_OPTIONS_GROUP);
            String mountPoint = m.group(MNTTAB_MOUNT_POINT_GROUP);

            results.add(new FileSystemMount(server, device, mountPoint, type, options));
        }
        return results;
    }
    
    /**
     * Unit test hook
     */
    protected File getMnttabFile() {
        return new File("/etc/mnttab");
    }
    
    @Override
    public String getArch() {
        String jvmArch = super.getArch();
        if (jvmArch.toLowerCase().startsWith("pa")) {
            return "hppa";
        }
        else if (jvmArch.toLowerCase().startsWith("ia64")) {
            return "ia64";
        }
        return jvmArch;
    }

}

