/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/platform/linux/ProcDevicesFile.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.platform.linux;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.netapp.common.MsgKey;
import com.netapp.common.logging.NALogger;
import com.netapp.common.platform.PlatformException;
import com.netapp.common.util.Utils;

/**
 * Represent /proc/devices for looking up block device mappings.
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class ProcDevicesFile {
    private static final NALogger logger = NALogger.getLogger(ProcDevicesFile.class);
    
    public static final File SYSTEM_DEVICES_FILE = new File("/proc/devices");
    
    private static final Pattern CHAR_HEADER_PATTERN = Pattern.compile("Character devices:.*");
    private static final Pattern BLOCK_HEADER_PATTERN = Pattern.compile("Block devices:.*");
    private static final Pattern DEV_ENTRY = Pattern.compile("\\s*(\\d+)\\s+(\\S+)");

    private Map<Integer, String> blockDevices;
    private Map<Integer, String> charDevices;
    
    protected ProcDevicesFile() {
        this.blockDevices = new HashMap<Integer, String>();
        this.charDevices = new HashMap<Integer, String>();
    }
    
    protected ProcDevicesFile(File path) {
        this();
        Map<Integer, String> currentMap = null;
        try {
            BufferedReader bufferedReader = new BufferedReader(new FileReader(path));
            for (String line = bufferedReader.readLine(); line != null; line = bufferedReader.readLine()) {
                Matcher entryMatcher = DEV_ENTRY.matcher(line);
                if (currentMap != null && entryMatcher.matches()) {
                    int major = Integer.parseInt(entryMatcher.group(1));
                    String device = entryMatcher.group(2);
                    currentMap.put(major, device);
                }
                else if (CHAR_HEADER_PATTERN.matcher(line).matches()) {
                    currentMap = this.charDevices;
                }
                else if (BLOCK_HEADER_PATTERN.matcher(line).matches()) {
                    currentMap = this.blockDevices;
                }
            }
        }
        catch (IOException e) {
            String error = logger.error(MsgKey.PLATFORM_UNABLE_TO_LOAD_DEVICES_FILE, 
                e, path.getAbsolutePath(), Utils.prettyString(e));
            throw new PlatformException(error, e);
        }
    }
    
    public String driverFromBlockMajor(int major) {
        return blockDevices.get(major);
    }
    
    public String driverFromCharMajor(int major) {
        return charDevices.get(major);
    }
    
    // These are for unit tests
    public void addBlockDriver(int major, String module) {
        this.blockDevices.put(major, module);
    }
    
    public void addCharDriver(int major, String module) {
        this.charDevices.put(major, module);
    }
}
