/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/platform/unix/FileSecurityDescriptorImpl.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.platform.unix;

import java.util.HashSet;
import java.util.Set;

import com.netapp.common.platform.FileOwner;
import com.netapp.common.platform.FileSecurityDescriptor;
import com.netapp.common.platform.Group;
import com.netapp.common.platform.PosixAccessControlBitMap;
import com.netapp.common.platform.User;
import com.netapp.common.util.CollectionUtils;
import com.netapp.common.util.Transformer;

public class FileSecurityDescriptorImpl implements FileSecurityDescriptor {
    
    private FileOwner fileOwner;

    private int permissionBits;

    public FileSecurityDescriptorImpl() {   
    }

    /**
     * Used by the Unix JNI methods
     * @param user
     * @param group
     * @param permissionBits
     */
    FileSecurityDescriptorImpl(User user, Group group, int permissionBits) {
        this.fileOwner = new FileOwner(user, group);
        this.permissionBits = permissionBits;
    }
    
    public FileOwner getFileOwner() {
        return this.fileOwner;
    }

    public void setFileOwner(FileOwner fileOwner) {
        this.fileOwner = fileOwner;
    }

    public int getPermissionBits() {
        return this.permissionBits;
    }

    public String getPermissionOctal() {
        return Integer.toOctalString(this.permissionBits);
    }
    
    public void setPermissionBits(int permissionBits) {
        this.permissionBits = permissionBits;
    }

    /**
     *  This call toggles the corresponding permission bits to on.
     */
    public void addAccessControls(AccessControl... accessControls) {
        
        for (AccessControl accessControl : accessControls) {
            PosixAccessControlBitMap map = PosixAccessControlBitMap.lookup(accessControl);
        
            this.permissionBits |= map.getPermissionBit();
        }
    }

    /**
     * This call toggles the corresponding permission bits to off.
     */
    public void removeAccessControls(AccessControl... accessControls) {
        
        for (AccessControl accessControl : accessControls) {
            PosixAccessControlBitMap map = PosixAccessControlBitMap.lookup(accessControl);
        
            this.permissionBits &= ~map.getPermissionBit();
        }
    }

    public Set<AccessControl> getEnabledAccessControls() {
        return CollectionUtils.transform(PosixAccessControlBitMap.findMatches(this.permissionBits), new HashSet<AccessControl>(), new Transformer<PosixAccessControlBitMap, AccessControl>() {

            public AccessControl transform(PosixAccessControlBitMap s) {
                
                return s.getAccessControl();
            }
            
        });
    }
    
    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((this.fileOwner == null) ? 0 : this.fileOwner.hashCode());
        result = prime * result + this.permissionBits;
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (!(obj instanceof FileSecurityDescriptorImpl)) {
            return false;
        }
        final FileSecurityDescriptorImpl other = (FileSecurityDescriptorImpl) obj;
        if (this.fileOwner == null) {
            if (other.fileOwner != null) {
                return false;
            }
        }
        else if (!this.fileOwner.equals(other.fileOwner)) {
            return false;
        }
        if (this.permissionBits != other.permissionBits) {
            return false;
        }
        return true;
    }

    @Override
    public String toString() {
        return this.fileOwner + " " + Integer.toOctalString(this.permissionBits);
    }
}
