/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/platform/windows/WindowsAuthenticatorNativeImpl.java#1 $
 *
 * Copyright (c) 2008 Network Appliance, Inc.
 * All rights reserved.
 *
 */
package com.netapp.common.platform.windows;

import com.netapp.common.logging.NALogger;
import com.netapp.common.platform.Authenticator;
import com.netapp.common.util.ConfigBundle;

public class WindowsAuthenticatorNativeImpl implements Authenticator {

    private static NALogger logger = NALogger.getLogger();

    native boolean nativeAuthenticate(String domain, String user, String password);

    static {
        System.loadLibrary("com_netapp_common_platform_windows_WindowsAuthenticatorNativeImpl");
    }

    protected String errorMessage;
    
    protected ConfigBundle configBundle;

    public WindowsAuthenticatorNativeImpl(ConfigBundle configBundle) {
        this.configBundle = configBundle;
    }

    public boolean authenticate(String username, String password) {
        setErrorMessage(null);
        
        String jniDomain = ".";
        String jniUser = new String(username);
        String jniPassword = new String(password);        
        
        if (username.indexOf("\\") > 0) {
            final String[] temp = username.split("\\\\");
            jniDomain = new String(temp[0]);
            jniUser = new String(temp[1]);                
        }        
        
        try {
            logger.internal("authenticate: Calling JNI authenticateUser(%s,%s)", jniDomain, jniUser);
            if (this.nativeAuthenticate(jniDomain, jniUser, jniPassword)) {
                logger.internal("authenticate: User '%s' granted access", username);
                return true;
            }
            else {
                logger.internal("authenticate: User '%s' NOT granted access", username);
                return false;
            }
        }
        // CHECKSTYLE:OFF         
        catch (Exception e) {
            // CHECKSTYLE:ON 
            setErrorMessage(e.getLocalizedMessage());
            logger.internal("authenticate: exception caught '%s', '%s'", e, e.getLocalizedMessage());
        }

        return false;
    }

    public void setErrorMessage(String msg) {
        if (msg == null) {
            this.errorMessage = null;
        }
        else {
            StringBuffer sb = new StringBuffer(msg);
            this.errorMessage = sb.toString();
        }
    }

    public String getErrorMessage() {
        return this.errorMessage;
    }

    public static void main(String[] args) {
        if (args.length < 2) {
            System.out.println("WindowsAuthenticatorNativeImpl syntax:\n\n\t<domain>\\<username> <password>\n\n\t\tOR\n\n\t<username> <password>  ... acts as if \".\\username\" was provided");
            System.exit(1);
        }

        String username = args[0];
        String password = args[1];
        WindowsAuthenticatorNativeImpl authenticator = new WindowsAuthenticatorNativeImpl(new ConfigBundle());
        System.out.println("Authenticated ['" + authenticator.authenticate(username, password) + "']\terrorString['" + authenticator.getErrorMessage() + "']");
    }
}
