/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/util/ErrorCodeUtils.java#1 $
 * 
 * Copyright (c) 2008 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.common.util;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Utility routines for parsing  APPL-01234 codes.
 * @author claird
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class ErrorCodeUtils {
    
    protected ErrorCodeUtils() {
    }

    /**
     * Regex for APPL-01234 codes.
     */
    static final Pattern CODE_PATTERN = Pattern.compile("^((\\w+)-(\\d+))(: (.*))?", Pattern.MULTILINE | Pattern.DOTALL);

    /**
     * For a message like 'PLAT-00902: Test parameterized messages: "{0}" and "{1}', 
     * this function will return the 902 portion of it.
     */
    public static Integer extractIntegerCode(String localizedMessage) {
        Matcher m = CODE_PATTERN.matcher(localizedMessage);
        if (!m.matches()) {
            return null;
        }
        String stringCode = m.group(3);
        return Integer.valueOf(stringCode);
    }
    
    /**
     * For a message like 'PLAT-00902: Test parameterized messages: "{0}" and "{1}', 
     * this function will return the string without "PLAT-00902: ".
     */
    public static String extractMessage(String localizedMessage) {                       
        String message = localizedMessage;
        Matcher m = CODE_PATTERN.matcher(localizedMessage);
        if (m.matches() && m.group(5) != null) {
            message = m.group(5);
        }
        return message;                
    }
}
