/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/util/SysInfo.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Modifier;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.Enumeration;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.TreeMap;
import java.util.TreeSet;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;

import org.apache.log4j.Appender;
import org.apache.log4j.FileAppender;
import org.apache.log4j.Logger;

import com.netapp.common.execution.ShellCommand;
import com.netapp.common.execution.ShellExecutor;
import com.netapp.common.execution.ShellExecutorFactory;
import com.netapp.common.execution.ShellResult;
import com.netapp.common.flow.FlowConstants;
import com.netapp.common.flow.TaskInput;
import com.netapp.common.flow.TaskUtils;
import com.netapp.common.logging.NALogger;
import com.netapp.common.platform.OperatingSystem;
import com.netapp.common.xml.sysinfo.Field;
import com.netapp.common.xml.sysinfo.OsFile;
import com.netapp.common.xml.sysinfo.SysinfoConfig;
import com.netapp.common.xml.sysinfo.Task;
import com.netapp.common.xml.sysinfo.TimeoutUnit;
import com.netapp.common.xml.sysinfo.Version;

/**
 * System based informations and details..
 * 
 * @author debert
 */
public class SysInfo {

    private static final String KEY_CONTENT_ENV_FILE = "sysinfo.diagnostic.file.content.env.file";

    private static final String KEY_CONTENT_ENV_TEXT = "sysinfo.diagnostic.file.content.env.text";
    private static final String KEY_CONTENT_EOF_TEXT = "sysinfo.diagnostic.file.content.eof.text";
    private static final String KEY_CONTENT_JVM_FILE = "sysinfo.diagnostic.file.content.jvm.file";

    private static final String KEY_CONTENT_JVM_TEXT = "sysinfo.diagnostic.file.content.jvm.text";
    private static final String KEY_CONTENT_SEPARATOR = "sysinfo.diagnostic.file.content.separator";
    private static final String KEY_CONTENT_STDERR_TEXT = "sysinfo.diagnostic.file.content.stderr.text";
    private static final String KEY_CONTENT_STDOUT_TEXT = "sysinfo.diagnostic.file.content.stdout.text";
    private static final String KEY_DATE_FORMAT = "sysinfo.diagnostic.date.format";
    private static final String KEY_DIAGNOSTIC_FILE_PATH = "sysinfo.diagnostic.file.path";
    private static final String KEY_FILE_IO_BUFFER = "sysinfo.diagnostic.file.io.buffer";
    private static final String KEY_OS_CMD_TIMEOUT = "sysinfo.diagnostic.os.cmd.timeout";
    private static final String KEY_SYSINFO_CONFIG_FILE = "sysinfo.diagnostic.config.file";
    /*
     * Logger
     */
    private static final NALogger logger = NALogger.getLogger(SysInfo.class);

    private String clientRequestId;
    /*
     * command prefix used for executing os commands
     */
    private final ConfigProperty<File> configFile = ConfigProperty.fileProperty(SysInfo.KEY_SYSINFO_CONFIG_FILE,
        this.defaultConfigFile);
    private final ConfigProperty<String> contentEnvText = ConfigProperty.stringProperty(SysInfo.KEY_CONTENT_ENV_TEXT,
        this.defaultContentENV);

    private final ConfigProperty<String> contentEofText = ConfigProperty.stringProperty(SysInfo.KEY_CONTENT_EOF_TEXT,
        this.defaultContentEOF);
    private final ConfigProperty<String> contentFileEnv = ConfigProperty.stringProperty(SysInfo.KEY_CONTENT_ENV_FILE,
        this.defaultContentFileENV);
    private final ConfigProperty<String> contentFileJvm = ConfigProperty.stringProperty(SysInfo.KEY_CONTENT_JVM_FILE,
        this.defaultContentFileJVM);
    private final ConfigProperty<String> contentJvmText = ConfigProperty.stringProperty(SysInfo.KEY_CONTENT_JVM_TEXT,
        this.defaultContentJVM);
    private final ConfigProperty<String> contentSeparator = ConfigProperty.stringProperty(
        SysInfo.KEY_CONTENT_SEPARATOR, this.defaultContentSeparator);
    private final ConfigProperty<String> contentStdErr = ConfigProperty.stringProperty(SysInfo.KEY_CONTENT_STDERR_TEXT,
        this.defaultContentStdErr);
    private final ConfigProperty<String> contentStdOut = ConfigProperty.stringProperty(SysInfo.KEY_CONTENT_STDOUT_TEXT,
        this.defaultContentStdOut);
    /*
     * application context
     */
    private ConfigBundle context;
    private final ConfigProperty<String> dateFormat = ConfigProperty.stringProperty(SysInfo.KEY_DATE_FORMAT,
        this.defaultDateFormat);
    private final File defaultConfigFile = new File("sysinfo-config.xml");

    private final String defaultContentENV = "Environment Variables";
    private final String defaultContentEOF = "END OF FILE";
    private final String defaultContentFileENV = "env.txt";
    private final String defaultContentFileJVM = "jvm.txt";
    private final String defaultContentJVM = "Java Virtual Machine settings";
    private final String defaultContentSeparator = "\n===== %s =====\n";
    private final String defaultContentStdErr = "STDERR";
    private final String defaultContentStdOut = "STDOUT";
    private final String defaultDateFormat = "yyyy/MM/dd hh:mm:ss,SSS a 'GMT'Z";
    private final File defaultDiagnosticFilePath = new File("diagnostic");
    private final int defaultFileIoBuffer = 8 * 1024;
    /*
     * builtin defaults
     */
    private final long defaultOsCmdTimeout = 5 * 1000;
    private final ConfigProperty<File> diagPath = ConfigProperty.fileProperty(SysInfo.KEY_DIAGNOSTIC_FILE_PATH,
        this.defaultDiagnosticFilePath);
    private final ConfigProperty<Integer> fileIoBuffer = ConfigProperty.intProperty(SysInfo.KEY_FILE_IO_BUFFER,
        this.defaultFileIoBuffer);

    /*
     * operation specific parameters
     */
    private String operationId;
    private String operationType;
    private final ConfigProperty<Long> osCmdTimout = ConfigProperty.longProperty(SysInfo.KEY_OS_CMD_TIMEOUT,
        this.defaultOsCmdTimeout);

    private ZipOutputStream sysInfoDiagnosticFile;
    /*
     * file for output and output stream
     */
    private File sysInfoFile;

    private SysinfoConfig xmlSysInfoConfig;
    
    /*
     * Reference to JAXB to allow marshal/unmarshal of top level schema.
     */
    private JAXBContext jaxb;

    private boolean xmlValid;

    /*
     * map for zip toc administration 
     */
    TreeSet<String> zipToc = new TreeSet<String>();

    /**
     * Default Constructor Creates a new zip file "netapp_support.zip"
     * 
     * @throws FileNotFoundException
     */
    public SysInfo() throws IOException {
        initialize();
    }

    /**
     * Add version information from a static defined fields of a given class
     * 
     * @param className Class name.
     * @param fieldNames List of static fields
     * @throws IOException
     */
    public void add(final String className, final List<String> fieldNames) throws IOException {
        String currentFieldName = null;

        try {
            final Class<?> clazz = Class.forName(className);

            writeSeparator(className);

            for (final String fieldName : fieldNames) {
                currentFieldName = fieldName;

                final java.lang.reflect.Field field = clazz.getField(fieldName);

                if (Modifier.isStatic(field.getModifiers())) {
                    final Object obj = field.get(null);
                    this.sysInfoDiagnosticFile.write((fieldName + "=" + obj + "\n").getBytes());
                }
            }
        }
        catch (final ClassNotFoundException ex) {
            SysInfo.logger.internal("cannot load class " + className + " : " + ex.getMessage());
        }
        catch (final NoSuchFieldException ex) {
            SysInfo.logger.internal("cannot process field " + currentFieldName + " of " + className + " : "
                + ex.getMessage());
        }
        catch (final IllegalAccessException ex) {
            SysInfo.logger.internal("cannot process field " + currentFieldName + " of " + className + " : "
                + ex.getMessage());
        }
    }

    /**
     * Add a zip file entry.
     * 
     * @param description Description of the file / file content.
     * @param filename Name of the file
     * @param properties Properties to be dumped.
     * @throws IOException
     */
    public void add(final String description, final String filename, final Properties properties) throws IOException {
        final File file = new File(filename);
        final ZipEntry entry = getZipEntry(file.getName());

        final TreeMap<String, String> map = new TreeMap<String, String>();

        this.sysInfoDiagnosticFile.putNextEntry(entry);
        writeHeading(description, null);

        for (final Map.Entry<Object, Object> e : properties.entrySet()) {
            map.put((String) e.getKey(), (String) e.getValue());
        }

        for (final Map.Entry<String, String> m : map.entrySet()) {
            final byte[] b = String.format("%s=%s\n", m.getKey(), m.getValue()).getBytes();
            this.sysInfoDiagnosticFile.write(b, 0, b.length);

        }

        this.sysInfoDiagnosticFile.closeEntry();
    }

    /**
     * Add a zip file entry.
     * 
     * @param description Description of the file / file content.
     * @param filename Name of the file.
     * @param map Map to be dumped.
     * @throws IOException
     */
    public void add(final String description, final String filename, final TreeMap<String, String> map)
        throws IOException {
        final File file = new File(filename);
        final ZipEntry entry = getZipEntry(file.getName());

        this.sysInfoDiagnosticFile.putNextEntry(entry);
        writeHeading(description, null);

        for (final Map.Entry<String, String> e : map.entrySet()) {
            final byte[] b = String.format("%s=%s\n", e.getKey(), e.getValue()).getBytes();
            this.sysInfoDiagnosticFile.write(b, 0, b.length);
        }
        this.sysInfoDiagnosticFile.closeEntry();
    }

    /**
     * Close and finalize the zip file.
     * 
     * @throws IOException
     */
    public void close() throws IOException {
        SysInfo.logger.internal("sysinfo: closing zip file");
        if (this.sysInfoDiagnosticFile != null) {
            this.sysInfoDiagnosticFile.close();
        }
    }

    /**
     * Dump commands and files to the same zip file.
     * 
     * @param commands List of commands.
     * @param files List of files.
     * @throws IOException
     */
    public void dump(final List<String> commands, final List<String> files) throws IOException {
        if (commands != null) {
            for (final String cmd : commands) {
                dumpOsCommand(cmd, null, 0);
            }
        }

        if (files != null) {
            for (final String file : files) {
                dumpFile(file);
            }
        }
    }

    /**
     * Dump environment.
     * 
     * @throws IOException
     */
    public void dumpEnv() throws IOException {
        SysInfo.logger.internal("sysinfo: dump env properties");
        add(this.contentEnvText.lookup(this.context), this.contentFileEnv.lookup(this.context),
            new TreeMap<String, String>(System.getenv()));
    }

    /**
     * Dump a file.
     * 
     * @param filename File to be dumped.
     * @throws IOException
     */
    public void dumpFile(final String filename) throws IOException {
        add(new File(filename));
    }

    /**
     * Dump all os commands defined in a xml configuration file.
     * 
     * @throws IOException
     */
    public void dumpFromXML() throws IOException {
        long timeout = 0;

        if (!isXmlValid()) {
            SysInfo.logger.internal("no valid xml configuration found");
            return;
        }

        final List<com.netapp.common.xml.sysinfo.OsCommand> listOsCommands = this.xmlSysInfoConfig.getOsCommands();
        OperatingSystem operatingSystem = OperatingSystem.createInstance();
        
        logger.internal("Found %d os commands to run", listOsCommands.size());

        if (listOsCommands != null) {
            for (final com.netapp.common.xml.sysinfo.OsCommand osCommand : listOsCommands) {
                /*
                 * get os command specific timeout
                 * if 0, no timeout was specified, take the default one
                 */
                if ((timeout = osCommand.getTimeoutValue()) > 0) {
                    if (osCommand.getTimeoutUnit().equals(TimeoutUnit.S)) {
                        timeout *= 1000;
                    }
                }
                else {
                    timeout = this.osCmdTimout.lookup(this.context);
                }
                
                logger.internal("Command '%s' has %d tasks to attempt to run", osCommand.getDescription(), osCommand.getCommands().size());

                for (Task osTask : osCommand.getCommands()) {
                    /*
                     * Verify that the current command is valid for the current OS
                     */
                    logger.internal("Checking task platform of '%s'", osTask.getPlatform());
                    if (!StringUtils.isEmpty(osTask.getPlatform()) 
                            && !operatingSystem.getPlatform().toLowerCase().startsWith(osTask.getPlatform().toLowerCase())) {
                        logger.internal("Task platform '%s' does not equal current host platform of '%s'", osTask.getPlatform(), operatingSystem.getPlatform());
                        continue;
                    }
                    
                    logger.internal("Checking task version of '%s'", osTask.getVersion());
                    if (!StringUtils.isEmpty(osTask.getVersion()) 
                            && !operatingSystem.getVersion().toLowerCase().startsWith(osTask.getVersion().toLowerCase())) {
                        logger.internal("Task host version '%s' does not equal current host version of '%s'", osTask.getVersion(), operatingSystem.getVersion());
                        continue;
                    }

                    /*
                     * dump the task
                     */
                    String cmdTask = null;

                    if ((osTask != null) && ((cmdTask = osTask.getTask()) != null)) {
                        dumpOsCommand(cmdTask, osCommand.getDescription(), timeout);
                    }
                }
            }
        }

        /*
         * dump files
         */
        final List<OsFile> listOsFiles = this.xmlSysInfoConfig.getOsFiles();
        if (listOsFiles != null) {
            for (final OsFile osFile : listOsFiles) {
                final File logFile = new File(osFile.getPath(), osFile.getFile());
                SysInfo.logger.internal("sysinfo: processing file: " + logFile.getAbsolutePath());
                dumpFile(logFile.getAbsolutePath());
            }
        }
    }

    /**
     * Dump Java Virtual Machine information.
     * 
     * @throws IOException
     */
    public void dumpJvmInfo() throws IOException {
        SysInfo.logger.internal("sysinfo: dump jvm properties");
        add(this.contentJvmText.lookup(this.context), this.contentFileJvm.lookup(this.context), System.getProperties());
    }

    /**
     * Dump log files generated from defined/used log4j appenders.
     */
    public void dumpLogFiles() {
        final Logger logDump = Logger.getRootLogger();
        // CHECKSTYLE:OFF
        final Enumeration enumAppender = logDump.getAllAppenders();
        // CHECKSTYLE:ON
        while (enumAppender.hasMoreElements()) {
            final Appender appender = (Appender) enumAppender.nextElement();
            if (appender instanceof FileAppender) {
                final FileAppender fileAppender = (FileAppender) appender;
                SysInfo.logger.internal("sysinfo: processing log file: " + fileAppender.getFile());
                try {
                    dumpFile(fileAppender.getFile());
                }
                catch (final IOException ex) {
                    SysInfo.logger.internal("sysinfo: unable to process log file: " + fileAppender.getFile());
                }
            }
        }
    }

    /**
     * Dump Standard Output / Error from OS Command execution
     * 
     * @param command Command to be executed.
     * @param description A string value that represents the description to the command. If <b>null</b> the description
     *        is created from the command name and parameters.
     * @param timeout A long value that represents the timeout for the command.
     * @throws IOException
     */
    public void dumpOsCommand(final String command, final String description, final long timeout) throws IOException {
        // final List<String> cmd = new LinkedList<String>(Arrays.asList(this.cmdPrefix));
        // cmd.add(command);
        //
        // final ProcessBuilder builder = new ProcessBuilder(cmd);

        final long osCommandTimeout = timeout <= 0 ? this.osCmdTimout.lookup(this.context) : timeout;
        String zipFileName = command.replaceAll(" ", "");
        zipFileName = zipFileName.replaceAll("-", "_");
        zipFileName = zipFileName.replaceAll("/", "_");
        zipFileName = zipFileName.replaceAll("\\\\", "_");
        // zipFileName = zipFileName.replaceAll(File.separator, "_"); // This has issues with Windows '\' file separator
        zipFileName = zipFileName.concat(".txt");

        ShellExecutor executor = ShellExecutorFactory.createShellExecutor();
        ShellCommand osCommand = new ShellCommand(command);
        ShellResult result = executor.execute(osCommand, osCommandTimeout);
        final int osResult = result.getExitValue();

        final String osCommandDescr = description == null ? command : description;

        add(command, osCommandDescr, zipFileName, result);
    }

    /**
     * Dump the version information.
     * <P>
     * Contains the version of the package and all defined fields of classes from the XML configuration file.
     * 
     * @throws Exception
     */
    public void dumpVersion() throws Exception {
        final ZipEntry entry = getZipEntry("version.txt");
        this.sysInfoDiagnosticFile.putNextEntry(entry);

        writeHeading("Version Information", null);

        /*
         * builtin version
         */
        final String builtinClass = "com.netapp.common.Version";
        final String[] builtinFields = new String[] { "VERSION", "BUILD_DATE", "CHANGEID" };

        add(builtinClass, Arrays.asList(builtinFields));

        /*
         * dump version information 
         * source: xml configuration file
         */
        if (isXmlValid()) {
            final List<Version> listVersions = this.xmlSysInfoConfig.getVersions();
            for (final Version version : listVersions) {
                final List<String> fieldNames = new LinkedList<String>();
                for (final Field field : version.getFields()) {
                    fieldNames.add(field.getName());
                }

                add(version.getClass().getName(), fieldNames);
                fieldNames.clear();
            }
        }

        this.sysInfoDiagnosticFile.closeEntry();

    }

    /**
     * Open a zip compressed file for output
     * <p>
     * If the file already exists, it will be renamed to the same name and path, with the current timestamp added
     * 
     * @throws FileNotFoundException
     */
    public void open() throws FileNotFoundException {
        if ((this.sysInfoFile = TaskUtils.configValue(FlowConstants.DIAGNOSTIC_OUTPUT_PATH)) == null) {
            this.sysInfoFile = this.diagPath.lookup(this.context);
        }

        if (!this.sysInfoFile.exists()) {
            if (!this.sysInfoFile.mkdirs()) {
                throw new FileNotFoundException("unable to create diagnostic directory "
                    + this.sysInfoFile.getAbsolutePath());
            }
        }

        String fileName = null;
        if (this.clientRequestId != null) {
            fileName = String.format("%s_%s_%s.zip", this.operationType, this.operationId, this.clientRequestId);
        }
        else {
            fileName = String.format("%s_%s.zip", this.operationType, this.operationId);
        }
        this.sysInfoFile = new File(this.sysInfoFile, fileName);

        /* 
         * rename an existing file
         */
        if (this.sysInfoFile.exists()) {
            final String tmp = String.format("%s_%d", this.sysInfoFile.getAbsoluteFile(), System.currentTimeMillis());
            final File renameTo = new File(tmp);
            SysInfo.logger.internal(String.format("sysinfo: renaming existing %s to %s", this.sysInfoFile
                .getAbsolutePath(), renameTo.getAbsolutePath()));
            this.sysInfoFile.renameTo(renameTo);
        }

        this.sysInfoDiagnosticFile = new ZipOutputStream(new FileOutputStream(this.sysInfoFile, false));
        if (this.sysInfoDiagnosticFile == null) {
            SysInfo.logger.internal("sysinfo: unable to create/open diagnostic file");
            throw new FileNotFoundException("unable to create/open diagnostic file");
        }
    }

    /**
     * Set the client request id.
     * 
     * @param clientRequestId
     */
    @TaskInput
    public void setClientRequestId(final String clientRequestId) {
        this.clientRequestId = clientRequestId;
    }

    /**
     * Set the id of the operation.
     * 
     * @param operationId
     */
    @TaskInput
    public void setOperationId(final String operationId) {
        this.operationId = operationId;
    }

    /**
     * Set the operation type of the operation.
     * 
     * @param operationType
     */
    @TaskInput
    public void setOperationType(final String operationType) {
        this.operationType = operationType;
    }

    /**
     * Add a file
     * 
     * @param file File or directory.
     * @return true, if file or directory exist and is readable.
     * @throws IOException
     */
    private boolean add(final File file) throws IOException {
        SysInfo.logger.internal("sysinfo: add file: " + file.getAbsolutePath());

        if (!(file.exists() && file.canRead())) {
            return false;
        }

        if (file.isFile()) {
            try {
                addSingleFile(file);
            }
            catch (final IOException ex) {
                SysInfo.logger.internal("sysinfo: unable to open file " + file.getAbsolutePath());
            }
        }
        else if (file.isDirectory()) {
            for (final File f : file.listFiles()) {
                add(f);
            }
        }
        return true;
    }

    /**
     * Add a zip file entry.
     * 
     * @param command OS command.
     * @param description Description of the file.
     * @param filename Name of the file.
     * @param process Process information, standard out and standard error from os command execution.
     * @throws IOException
     */
    private void add(final String command, final String description, final String filename,
        final ShellResult shellResult) throws IOException {
        final ZipEntry entry = getZipEntry(filename);

        this.sysInfoDiagnosticFile.putNextEntry(entry);
        writeHeading(description, command);

        if (CollectionUtils.isEmpty(shellResult.getStdOut())) {
            writeSeparator(this.contentStdOut.lookup(this.context));
            writeList(shellResult.getStdOut());
        }

        if (CollectionUtils.isEmpty(shellResult.getStdErr())) {
            writeSeparator(this.contentStdErr.lookup(this.context));
            writeList(shellResult.getStdErr());
        }

        writeSeparator(this.contentEofText.lookup(this.context));

        this.sysInfoDiagnosticFile.closeEntry();
    }

    /**
     * Add a single file
     * 
     * @param file File to be added.
     * @throws IOException
     */
    private void addSingleFile(final File file) throws IOException {
        final ZipEntry entry = getZipEntry(file.getName());

        final FileInputStream inFile = new FileInputStream(file);

        this.sysInfoDiagnosticFile.putNextEntry(entry);

        writeStream(inFile);

        this.sysInfoDiagnosticFile.closeEntry();
    }

    private ZipEntry getZipEntry(final String entry) {
        if (!this.zipToc.contains(entry)) {
            this.zipToc.add(entry);
            return new ZipEntry(entry);
        }

        String entryX = null;
        for (int i = 1; i < 1000; i++) {
            entryX = String.format("%s_%03d", entry, i++);
            if (!this.zipToc.contains(entryX)) {
                this.zipToc.add(entryX);
                return new ZipEntry(entryX);
            }
        }

        /*
         * return command_TimeInMillis
         */
        entryX = String.format("%s_%d", entry, System.currentTimeMillis());
        this.zipToc.add(entryX);
        return new ZipEntry(entryX);
    }

    /*
     * basic initialization
     */
    private void initialize() {
        this.context = TaskUtils.getApplicationContext();

        // system defined classpath
        final String classPath = System.getProperty("java.class.path");

        File sysinfoConfigPath = null;

        if (classPath != null) {
            for (final String s : classPath.split(File.pathSeparator)) {
                final File path = new File(s + File.separator + this.defaultConfigFile);
                if (path.exists() && path.canRead()) {
                    sysinfoConfigPath = path;
                    break;
                }
            }
        }

        if (sysinfoConfigPath == null) {
            sysinfoConfigPath = new File(this.diagPath.lookup(this.context) + File.separator
                + this.configFile.lookup(this.context));
        }

        SysInfo.logger.internal("configuration file: " + sysinfoConfigPath);

        this.xmlValid = openXmlConfig(sysinfoConfigPath);
    }

    private boolean isXmlValid() {
        return this.xmlValid;
    }

    private boolean openXmlConfig(final File inFile) {
        try {
            if (!inFile.exists()) {
                SysInfo.logger.internal("configuration file " + inFile.getAbsolutePath() + " does not exist");
                return false;
            }

            this.jaxb = JAXBContext.newInstance(SysinfoConfig.class);

            Unmarshaller unmarshaller = this.jaxb.createUnmarshaller();
            
            this.xmlSysInfoConfig = (SysinfoConfig) unmarshaller.unmarshal(inFile);
            
            return true;
            
        }
        catch (JAXBException e) {
            SysInfo.logger.internal("JAXB error (%s) reading config file %s", e.getMessage(), inFile.getPath());
        }

        return false;
    }

    /**
     * Add a heading on the top the file. The heading contains a description and a timestamp.
     * 
     * @param description Description of the file / file content.
     * @throws IOException
     */
    private void writeHeading(final String description, final String command) throws IOException {
        final SimpleDateFormat dateFormatter = new SimpleDateFormat(this.dateFormat.lookup(this.context));
        final String timestamp = dateFormatter.format(new Date());

        String heading = null;

        if (command == null) {
            heading = String.format("description : %s\n" + "timestamp   : %s\n"
                + "================================================================================\n", description,
                timestamp);
        }
        else {
            heading = String.format("description : %s\n" + "command     : %s\n" + "timestamp   : %s\n"
                + "================================================================================\n", description,
                command, timestamp);
        }

        this.sysInfoDiagnosticFile.write(heading.getBytes(), 0, heading.getBytes().length);
    }

    /**
     * Write a givenlList to the file.
     * 
     * @param in List to be written
     * @throws IOException
     */
    private void writeList(final List<String> in) throws IOException {
        if (in != null) {
            for (final String s : in) {
                final byte[] buffer = s.concat("\n").getBytes();
                this.sysInfoDiagnosticFile.write(buffer, 0, buffer.length);
            }
        }
    }

    /**
     * Add a separator inside a file content
     * 
     * @param separator
     * @throws IOException
     */
    private void writeSeparator(final String separator) throws IOException {
        final byte[] sep = String.format(this.contentSeparator.lookup(this.context), separator).getBytes();

        this.sysInfoDiagnosticFile.write(sep, 0, sep.length);
    }

    /**
     * Write a given stream to the file.
     * 
     * @param in Stream to be written.
     * @throws IOException
     */
    private void writeStream(final InputStream in) throws IOException {
        int len = 0;

        final byte[] buffer = new byte[this.fileIoBuffer.lookup(this.context)];

        if (in != null) {
            while ((len = in.read(buffer)) != -1) {
                this.sysInfoDiagnosticFile.write(buffer, 0, len);
            }
        }
    }
}
