/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/xml/CatalogEntityResolver.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.xml;

import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;

import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import com.netapp.common.MsgKey;
import com.netapp.common.logging.NALogger;
import com.netapp.common.util.StringUtils;
import com.netapp.common.xml.catalog.Catalog;
import com.netapp.common.xml.catalog.EntityType;

/**
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class CatalogEntityResolver implements EntityResolver {
    private static NALogger logger = NALogger.getLogger(CatalogEntityResolver.class);
    private JAXBContext jaxb;
    private List<Entity> entities;
    
    public CatalogEntityResolver() {
        this.entities = new ArrayList<Entity>();
        try {
            jaxb = JAXBContext.newInstance(Catalog.class);
        }
        catch (JAXBException e) {
            throw new RuntimeException(e);
        }
    }
    
    public void addCatalogs(String path) {
        try {        
            ClassLoader loader = Thread.currentThread().getContextClassLoader();
            Enumeration<URL> urls = loader.getResources(path);
            while (urls.hasMoreElements()) {
                addCatalog(urls.nextElement());
            }
        }
        catch (IOException e) {
            throw new RuntimeException(e);
        }

    }
    public synchronized void addCatalog(URL catalog) {
        ClassLoader loader = Thread.currentThread().getContextClassLoader();
        Unmarshaller u;
        try {
            u = jaxb.createUnmarshaller();
            Catalog catalogType = (Catalog) u.unmarshal(catalog);
            for (EntityType entry : catalogType.getEntity()) {
                Entity entity = new Entity();
                URL resource = loader.getResource(entry.getResource());
                if (resource == null) {
                    logger.error(MsgKey.XML_RESOLVER_NO_SUCH_RESOURCE, entry.getResource());
                    continue;
                }
                entity.setResource(resource);
                entity.setSuffix(entry.getSuffix());
                entity.setSystemId(entry.getSystemId());
                this.entities.add(entity);
            }
        }
        catch (JAXBException e) {
            throw new RuntimeException(e);
        }
    }
    
    /**
     * @see org.xml.sax.EntityResolver#resolveEntity(java.lang.String, java.lang.String)
     */
    public InputSource resolveEntity(String publicId, String systemId) throws SAXException, IOException {
        for (Entity entity : this.entities) {
            if (entity.matches(systemId)) {
                URL url = entity.getResource();
                logger.internal("resolveEntity(%s, %s) => %s", publicId, systemId, url);
                return new InputSource(url.openStream());
            }
        }
        logger.internal("resolveEntity(%s, %s) => UNKNOWN", publicId, systemId);
        return null;
    }

    private static class Entity {
        private String suffix;
        private String systemId;
        private URL resource;
        
        public Entity() {
        }
        
        public void setSuffix(String suffix) {
            this.suffix = suffix;
        }
        
        public void setSystemId(String systemId) {
            this.systemId = systemId;
        }
        
        public void setResource(URL resource) {
            this.resource = resource;
        }
        
        public URL getResource() {
            return this.resource;
        }
        
        public boolean matches(String systemId) {
            if (!StringUtils.isEmpty(this.systemId) && this.systemId.equals(systemId)) {
                return true;
            }
            
            if (!StringUtils.isEmpty(this.suffix) && systemId.endsWith(this.suffix)) {
                return true;
            }
            return false;
        }
    }
}
