/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/zapi/ZAPIHostInfo.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.zapi;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class ZAPIHostInfo {
    public static final int DEFAULT_PORT = -1;
    public static final String DEFAULT_PROTOCOL = "direct";
    
    protected static final Pattern URL_REGEX = Pattern.compile("(?:(proxy)://)?([^/@:]+)(?:/([^@]*))?(?:@([^:]+)(?:\\:(\\d+))?)?");
    
    protected static final int PROTO_GROUP = 1;
    protected static final int USER_GROUP = 2;
    protected static final int PASS_GROUP = 3;
    protected static final int HOST_GROUP = 4;
    protected static final int PORT_GROUP = 5;
    
    private String hostname;
    private int port;
    private String username;
    private String password;
    private boolean proxy;
    
    public ZAPIHostInfo() {
    }
    
    public ZAPIHostInfo(String url) {
        Matcher m = URL_REGEX.matcher(url);
        if (!m.matches()) {
            throw new IllegalArgumentException("Unable to parse ZAPI url " + url);
        }
        this.username = m.group(USER_GROUP);
        this.password = m.group(PASS_GROUP);
        this.hostname = m.group(HOST_GROUP);
        this.port = (m.group(PORT_GROUP) != null) ? Integer.parseInt(m.group(PORT_GROUP)) : DEFAULT_PORT;
        this.proxy = m.group(PROTO_GROUP) != null;
    }
    
    public ZAPIHostInfo(String hostname, int port, String username, String password, boolean proxy) {
        setHostname(hostname);
        setPort(port);
        setUsername(username);
        setPassword(password);
        setProxy(proxy);
    }

    public String getHostname() {
        return this.hostname;
    }

    public void setHostname(String hostname) {
        this.hostname = hostname;
    }

    public String getPassword() {
        return this.password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public int getPort() {
        return this.port;
    }

    public void setPort(int port) {
        this.port = port;
    }

    public String getUsername() {
        return this.username;
    }

    public void setUsername(String username) {
        this.username = username;
    }
    
    public boolean isProxy() {
        return this.proxy;
    }

    public void setProxy(boolean proxy) {
        this.proxy = proxy;
    }

    @Override
    public String toString() {
        StringBuffer buffer = new StringBuffer("ZAPIHostInfo: ");
        
        buffer.append("Hostname=");
        buffer.append(getHostname());
        buffer.append(", Port=");
        buffer.append(getPort());
        buffer.append(", Username=");
        buffer.append(getUsername());
        buffer.append(", Password=");
        buffer.append(getPassword());
        buffer.append(", isProxy=");
        buffer.append(isProxy());
        
        return buffer.toString();
    }
    
}